def __version__() -> str: ...
def evaluate(card1: str, card2: str, card3: str, card4: str, card5: str) -> int:
    """This function evaluates a poker hand of five cards.

    Cards are encoded as two characters:
    - one for symbol among "23456789TJQKA"
    - one for color among "CDHS"
    """

def evaluate_7(card1: str, card2: str, card3: str, card4: str, card5: str, card6: str, card7: str) -> int:
    """This function evaluates a poker hand of seven cards.

    Cards are encoded as two characters:
    - one for symbol among "23456789TJQKA"
    - one for color among "CDHS"
    """
